use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Database::Context;

use Xyzzy::Context -self;

field db => sub { shift->cfg->conn };

package Xyzzy::Database;

use Xyzzy::Handler -self;

use Xyzzy::Database::Config;

use DBI;

field db_type => sub { shift->cfg->db_type };
field db_name => sub { shift->cfg->db_name };
field db_user => sub { shift->cfg->db_user };
field db_pass => sub { shift->cfg->db_pass };
field db_options => sub { shift->cfg->db_options };

*db_datasource = *Xyzzy::Database::Config::db_datasource;

field conn => sub {
	my $self = shift;
	return DBI->connect($self->db_datasource, $self->db_user, $self->db_pass, $self->db_options);
};

sub db {
	return new Xyzzy::Database::Transaction(cfg => $self, txn => $self->conn);
}

sub handle {
	my $ctx = new Xyzzy::Database::Context(ctx => shift, cfg => $self);
	my $res = eval { super($ctx) };
	my $err = $@;
	my $db = $self->{conn};
	if($db && $db->{Executed}) {
		eval {
			$db->rollback;
			warn "rolling back database transaction\n"
				unless $err;
		};
		warn $@ if $@;
	}
	die $err if $err;
	return $res;
}
