use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Header;

use Clarity -self;

field vals;

sub new {
	my @vals = @_;

	shift if @_ & 1;
	while(@_) {
		my $key = shift;
		my $val = shift;
		confess "Malformed header token '$key'"
			if defined $key && !istoken($key);
	}

	return super(vals => \@vals); 
}

sub istoken() {
	return $_[0] =~ /^[!\#-'*.0-9A-Z^_\`a-z|-]+$/;
}

sub isnasty() {
	return $_[0] =~ /[^ -~]|^\s|\s$|\s\s|[;"]/;
}

sub toString {
	my @vals = @{$self->vals};

	my @out;

	push @out, shift @vals
		if @vals & 1;
	
	while(@vals) {
		my $key = shift @vals;
		my $val = shift @vals;
		my $out = defined $key ? $key : '';
		if(defined $val) {
			if(isnasty($val)) {
				$val =~ s/(["\\])/\\$1/g;
				$val = "\"$val\"";
			}
			
			$out .= '='
				if defined $key;

			$out .= $val;
		}
		push @out, $out;
	}

	return join('; ', @out);
}

use overload
	'""' => \&toString,
	fallback => 1;
